@@ -1,14 +1,13 @@
-use std::net::SocketAddr;
-use structopt::StructOpt;
-
 use anyhow::Result;
 use futures::prelude::*;
 use std::collections::BTreeMap;
-use std::io::{stdin, stdout, Write};
+use std::io::{stdout, Write};
 use std::sync::Arc;
 use std::time::Duration;
+use structopt::StructOpt;
+use termion::color::*;
+use termion::cursor::Goto;
 use termion::event::Key;
-use termion::input::TermRead;
 use termion::raw::IntoRawMode;
 use termion::screen::*;
 use termion_input_tokio::TermReadAsync;
@@ -16,6 +15,7 @@ use tokio::net::{tcp::OwnedReadHalf, TcpStream};
 use tokio::sync::Mutex;
 use tokio_serde::{formats::SymmetricalBincode, SymmetricallyFramed};
 use tokio_util::codec::{FramedRead, FramedWrite, LengthDelimitedCodec};
+use xenopeltis_common::Color;
 use xenopeltis_common::*;
 
 #[derive(StructOpt, Clone, Debug)]
@@ -26,7 +26,7 @@ pub struct Options {
 pub struct State {
     data: BTreeMap<Coordinate, Field>,
     data_dirty: BTreeMap<Coordinate, Field>,
-    game_state: GameStateMessage,
+    game_state: PlayerState,
     exit: bool,
 }
 
@@ -35,7 +35,7 @@ impl State {
         State {
             data: BTreeMap::new(),
             data_dirty: BTreeMap::new(),
-            game_state: GameStateMessage::Playing,
+            game_state: PlayerState::Playing,
             exit: false,
         }
     }
@@ -56,9 +56,9 @@ pub async fn handle_stream(state: Arc<Mutex<State>>, reader: OwnedReadHalf) -> R
                     .data_dirty
                     .insert(field_state.coordinate, field_state.field);
             }
-            Ok(Some(ServerMessage::GameState(game_state))) => {
+            Ok(Some(ServerMessage::PlayerState(player_state))) => {
                 let mut state_lock = state.lock().await;
-                state_lock.game_state = game_state;
+                state_lock.game_state = player_state.state;
             }
             _ => {
                 break;
@@ -73,13 +73,21 @@ pub async fn draw_task(state: Arc<Mutex<State>>) {
     draw_task_run(state).await.unwrap();
 }
 
+fn map_color(color: Color) -> String {
+    match color {
+        Color::Blue => Fg(Blue).to_string(),
+        Color::Cyan => Fg(Cyan).to_string(),
+        Color::Green => Fg(Green).to_string(),
+        Color::Magenta => Fg(Magenta).to_string(),
+        Color::Red => Fg(Red).to_string(),
+        Color::Yellow => Fg(Yellow).to_string(),
+    }
+}
+
 pub async fn draw_task_run(state: Arc<Mutex<State>>) -> Result<()> {
     let mut screen = AlternateScreen::from(stdout().into_raw_mode()?);
     write!(screen, "{}", termion::cursor::Hide)?;
-    screen.flush().unwrap();
-    let mut count: usize = 0;
-    let mut dirty: usize = 0;
-    let mut last_field: Field = Field::Empty;
+    screen.flush()?;
 
     let mut interval = tokio::time::interval(Duration::from_millis(20));
     loop {
@@ -89,30 +97,25 @@ pub async fn draw_task_run(state: Arc<Mutex<State>>) -> Result<()> {
             break;
         }
 
-        //write!(screen, "{}{}", termion::cursor::Goto(1, 1), count)?;
-        //write!(screen, "{}{}: {:?}", termion::cursor::Goto(1, 2), dirty, last_field)?;
-
         // draw dirty fields
         for (coordinate, field) in std::mem::take(&mut state_lock.data_dirty).iter() {
-            last_field = *field;
             let shape = match field {
-                Field::Empty => " ",
-                Field::Food => ".",
-                Field::Snake(_) => "X",
-                Field::Wall => "|",
+                Field::Empty => (None, "  "),
+                Field::Food(false) => (None, "🍏"),
+                Field::Food(true) => (None, "🍎"),
+                Field::Snake(color) => (Some(map_color(*color)), "██"),
+                Field::Wall => (None, "▒▒"),
             };
-            write!(
-                screen,
-                "{}{}",
-                termion::cursor::Goto(coordinate.col as u16 + 1, coordinate.row as u16 + 1),
-                shape
-            )?;
+            let goto = Goto(2 * coordinate.col as u16 + 1, coordinate.row as u16 + 1);
+            let reset = Fg(Reset);
+            match shape {
+                (None, s) => write!(screen, "{}{}", goto, s)?,
+                (Some(c), s) => write!(screen, "{}{}{}{}", goto, c, s, reset)?,
+            }
             state_lock.data.insert(*coordinate, *field);
-            dirty += 1;
         }
 
         screen.flush()?;
-        count += 1;
     }
 
     write!(screen, "{}", termion::cursor::Show)?;
@@ -137,7 +140,6 @@ async fn main() -> Result<()> {
 
     let draw_task = tokio::spawn(draw_task(state.clone()));
 
-    let stdin = stdin();
     let mut keys = tokio::io::stdin().keys_stream();
     loop {
         let key = keys.try_next().await?.unwrap();
@@ -147,6 +149,7 @@ async fn main() -> Result<()> {
                 state_lock.exit = true;
                 break;
             }
+            Key::Char('r') | Key::Char(' ') => framed.send(ClientMessage::Restart).await?,
             Key::Left | Key::Right | Key::Up | Key::Down => {
                 framed
                     .send(ClientMessage::Direction(DirectionMessage {
@@ -158,7 +161,7 @@ async fn main() -> Result<()> {
                             _ => unreachable!(),
                         },
                     }))
-                    .await;
+                    .await?;
             }
             _ => {}
         }
